@@ -32,7 +32,7 @@ namespace {
                      cling::ExternalInterpreterSource& source):
       ASTImporter(ToContext, ToFileManager, FromContext, FromFileManager,
                   MinimalImport), m_Source(source) {}
-    virtual ~ClingASTImporter() = default;
+    ~ClingASTImporter() override = default;
 
     void Imported(Decl *From, Decl *To) override {
       ASTImporter::Imported(From, To);
@@ -268,7 +268,7 @@ namespace cling {
         DeclarationName childDeclName = parentDecl->getDeclName();
         if (auto II = childDeclName.getAsIdentifierInfo()) {
           StringRef name = II->getName();
-          if (!name.empty() && name.startswith(filter))
+          if (!name.empty() && name.starts_with(filter))
             ImportDecl(parentDecl, childDeclName, childDeclName,
                        childDeclContext);
         }
