@@ -58,9 +58,18 @@ const std::unordered_map<std::string_view, std::string_view> typeTranslationMap{
    {"ULong64_t", "unsigned long long"},
    {"uint64_t", "std::uint64_t"}};
 
+// Natively supported types drop the default template arguments and the CV qualifiers in template arguments.
+// Any types used as a template argument of user classes will keep [U]Long64_t template arguments for the type alias,
+// e.g. MyClass<std::vector<Long64_t>> will normalize to `MyClass<std::vector<std::int64_t>>` but keep the original
+// spelling in the type alias.
+bool IsUserClass(const std::string &typeName)
+{
+   return typeName.rfind("std::", 0) != 0 && typeName.rfind("ROOT::VecOps::RVec<", 0) != 0;
+}
+
 // Recursively normalizes a template argument using the regular type name normalizer F as a helper.
 template <typename F>
-std::string GetNormalizedTemplateArg(const std::string &arg, F fnTypeNormalizer)
+std::string GetNormalizedTemplateArg(const std::string &arg, bool keepQualifier, F fnTypeNormalizer)
 {
    R__ASSERT(!arg.empty());
 
@@ -69,6 +78,9 @@ std::string GetNormalizedTemplateArg(const std::string &arg, F fnTypeNormalizer)
       return ROOT::Internal::GetNormalizedInteger(arg);
    }
 
+   if (!keepQualifier)
+      return fnTypeNormalizer(arg);
+
    std::string qualifier;
    // Type name template argument; template arguments must keep their CV qualifier
    if (arg.substr(0, 6) == "const " || (arg.length() > 14 && arg.substr(9, 6) == "const "))
@@ -118,6 +130,39 @@ std::vector<AnglePos> FindTemplateAngleBrackets(const std::string &typeName)
    return result;
 }
 
+template <typename F>
+void NormalizeTemplateArguments(std::string &templatedTypeName, F fnTypeNormalizer)
+{
+   const auto angleBrackets = FindTemplateAngleBrackets(templatedTypeName);
+   R__ASSERT(!angleBrackets.empty());
+
+   std::string normName;
+   std::string::size_type currentPos = 0;
+   for (std::size_t i = 0; i < angleBrackets.size(); i++) {
+      const auto [posOpen, posClose] = angleBrackets[i];
+      // Append the type prefix until the open angle bracket.
+      normName += templatedTypeName.substr(currentPos, posOpen + 1 - currentPos);
+
+      const auto argList = templatedTypeName.substr(posOpen + 1, posClose - posOpen - 1);
+      const auto templateArgs = ROOT::Internal::TokenizeTypeList(argList);
+      R__ASSERT(!templateArgs.empty());
+
+      const bool isUserClass = IsUserClass(templatedTypeName);
+      for (const auto &a : templateArgs) {
+         normName += GetNormalizedTemplateArg(a, isUserClass, fnTypeNormalizer) + ",";
+      }
+
+      normName[normName.size() - 1] = '>';
+      currentPos = posClose + 1;
+   }
+
+   // Append the rest of the type from the last closing angle bracket.
+   const auto lastClosePos = angleBrackets.back().second;
+   normName += templatedTypeName.substr(lastClosePos + 1);
+
+   templatedTypeName = normName;
+}
+
 } // namespace
 
 std::string ROOT::Internal::GetCanonicalTypePrefix(const std::string &typeName)
@@ -233,35 +278,44 @@ std::string ROOT::Internal::GetRenormalizedTypeName(const std::string &metaNorma
       return canonicalTypePrefix;
    }
 
-   const auto angleBrackets = FindTemplateAngleBrackets(canonicalTypePrefix);
-   R__ASSERT(!angleBrackets.empty());
-
-   std::string normName;
-   std::string::size_type currentPos = 0;
-   for (std::size_t i = 0; i < angleBrackets.size(); i++) {
-      const auto [posOpen, posClose] = angleBrackets[i];
-      // Append the type prefix until the open angle bracket.
-      normName += canonicalTypePrefix.substr(currentPos, posOpen + 1 - currentPos);
-
-      const auto argList = canonicalTypePrefix.substr(posOpen + 1, posClose - posOpen - 1);
-      const auto templateArgs = TokenizeTypeList(argList);
-      R__ASSERT(!templateArgs.empty());
-
-      for (const auto &a : templateArgs) {
-         normName += GetNormalizedTemplateArg(a, GetRenormalizedTypeName) + ",";
-      }
-
-      normName[normName.size() - 1] = '>';
-      currentPos = posClose + 1;
-   }
-
-   // Append the rest of the type from the last closing angle bracket.
-   const auto lastClosePos = angleBrackets.back().second;
-   normName += canonicalTypePrefix.substr(lastClosePos + 1);
+   std::string normName{canonicalTypePrefix};
+   NormalizeTemplateArguments(normName, GetRenormalizedTypeName);
 
    return normName;
 }
 
+bool ROOT::Internal::NeedsMetaNameAsAlias(const std::string &metaNormalizedName, std::string &renormalizedAlias,
+                                          bool isArgInTemplatedUserClass)
+{
+   const auto canonicalTypePrefix = ROOT::Internal::GetCanonicalTypePrefix(metaNormalizedName);
+   if (canonicalTypePrefix.find('<') == std::string::npos) {
+      // If there are no templates, the function is done.
+      return false;
+   }
+
+   bool result = false;
+   const bool hasTemplatedUserClassParent = isArgInTemplatedUserClass || IsUserClass(canonicalTypePrefix);
+   auto fnCheckLong64 = [&](const std::string &arg) -> std::string {
+      if ((arg == "Long64_t" || arg == "ULong64_t") && hasTemplatedUserClassParent) {
+         result = true;
+         return arg;
+      }
+
+      std::string renormalizedArgAlias;
+      if (NeedsMetaNameAsAlias(arg, renormalizedArgAlias, hasTemplatedUserClassParent)) {
+         result = true;
+         return renormalizedArgAlias;
+      }
+
+      return GetRenormalizedTypeName(arg);
+   };
+
+   renormalizedAlias = canonicalTypePrefix;
+   NormalizeTemplateArguments(renormalizedAlias, fnCheckLong64);
+
+   return result;
+}
+
 std::string ROOT::Internal::GetNormalizedUnresolvedTypeName(const std::string &origName)
 {
    const TClassEdit::EModType modType = static_cast<TClassEdit::EModType>(
@@ -280,8 +334,7 @@ std::string ROOT::Internal::GetNormalizedUnresolvedTypeName(const std::string &o
    R__ASSERT(!angleBrackets.empty());
 
    // For user-defined class types, we will need to get the default-initialized template arguments.
-   const bool isUserClass =
-      (canonicalTypePrefix.substr(0, 5) != "std::") && (canonicalTypePrefix.substr(0, 19) != "ROOT::VecOps::RVec<");
+   const bool isUserClass = IsUserClass(canonicalTypePrefix);
 
    std::string normName;
    std::string::size_type currentPos = 0;
@@ -295,7 +348,7 @@ std::string ROOT::Internal::GetNormalizedUnresolvedTypeName(const std::string &o
       R__ASSERT(!templateArgs.empty());
 
       for (const auto &a : templateArgs) {
-         normName += GetNormalizedTemplateArg(a, GetNormalizedUnresolvedTypeName) + ",";
+         normName += GetNormalizedTemplateArg(a, isUserClass, GetNormalizedUnresolvedTypeName) + ",";
       }
 
       // For user-defined classes, append default-initialized template arguments.
@@ -314,7 +367,8 @@ std::string ROOT::Internal::GetNormalizedUnresolvedTypeName(const std::string &o
             R__ASSERT(expandedTemplateArgs.size() >= templateArgs.size());
 
             for (std::size_t j = templateArgs.size(); j < expandedTemplateArgs.size(); ++j) {
-               normName += GetNormalizedTemplateArg(expandedTemplateArgs[j], GetNormalizedUnresolvedTypeName) + ",";
+               normName +=
+                  GetNormalizedTemplateArg(expandedTemplateArgs[j], isUserClass, GetNormalizedUnresolvedTypeName) + ",";
             }
          }
       }
